@@ -1,3 +1,3 @@
 utils::globalVariables(c("id","name","symbol","slug","rank","is_active","first_historical_data","last_historical_data","timestamp",
-                         "close_ratio","coins","hist_date","history_url","market","Date","Name","Symbol","ref_cur",
+                         "close_ratio","coins","hist_date","history_url","market","Date","Name","Symbol","ref_cur","historyurl","finalWait",
                          "name","name_main","platform_locale","slug","slug_main","symbol","value","volume",".","code","sign"))
